"""CheckIPPlugin
Copyright (c) 2008, Piotr Furman
License: BSD
"""

from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import manage_users

from Globals import InitializeClass
from OFS.Cache import Cacheable
from zExceptions import Unauthorized

from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import classImplements
from Products.PluggableAuthService.interfaces.plugins import IAuthenticationPlugin
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from config import DISABLE, ALLOWED

try:
    from IPy import IP
    IP_ENABLED = True
except ImportError:
    # hide IP with this function if IPy is not available
    IP = lambda x: (x,)
    IP_ENABLED = False

manage_addCheckIPPluginForm = PageTemplateFile(
    'www/checkipAdd', globals(), __name__='manage_addCheckIPPluginForm' )

def manage_addCheckIPPlugin(dispatcher, id, title=None, REQUEST=None):
    """ Add a CheckIPPlugin to a Pluggable Auth Service user folder."""

    obj = CheckIPPlugin(id, title)
    dispatcher._setObject(obj.getId(), obj)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(
            '%s/manage_workspace?manage_tabs_message=CheckIPPlugin+added.' % dispatcher.absolute_url())

class CheckIPPlugin(BasePlugin, Cacheable):
    """ This PAS plugin allows to log in only for particular ip addresses. 

        List of allowed addresses was divided into two areas just for performance reasons 
        (one "for" loop less)
    """

    meta_type = 'CheckIPPlugin'

    security = ClassSecurityInfo()
    security.declareProtected(manage_users, 'manage_ip_lists')

    manage_ip_lists = PageTemplateFile('www/checkipManage', globals())
    manage_options = ({'label':'Allowed IP', 'action':'manage_ip_lists'},) + BasePlugin.manage_options

    def __init__(self, id, title=None):
        self._id = self.id = id
        self.title = title
        self._ip_list = []      # this list contains only single ip
        self._net_list = []     # this may contain subnets (ip with mask)


    security.declarePrivate('authenticateCredentials')
    def authenticateCredentials(self, credentials):
        """ This function is called when user tries to log in. 
            Returns None for allowed ip, and raise an exception for the other
        """

        # do nothing if disabled in config 
        if DISABLE:
            return None

        # get ip address
        client = self.REQUEST.get('HTTP_X_FORWARDED_FOR', None)
        if client:
            client = client.split(',')[0].strip()
        else:
            client = self.REQUEST.get('REMOTE_ADDR', None)
        # if for some reason request doesn't contain ip raise an exception
        if not client:
            raise Unauthorized

        # check always allowed list
        if client in ALLOWED:
            return None

        # for first list perform simple check, if ip is in list
        if client in self._ip_list:
            return None

        # for second list check each subnet
        for ip in self._net_list:
            if client in ip:
                return None

        # if we're still here client ip was not accepted
        raise Unauthorized

    security.declareProtected(manage_users, 'manage_setCheckIPLists')
    def manage_setCheckIPLists(self, ip_list='', net_list='', REQUEST=None):
        """ Set both ip lists """
        self._ip_list = [ip.strip() for ip in ip_list.split(u"\n") if ip]
        self._net_list = [IP(ip.strip()) for ip in net_list.split(u"\n") if ip]

        if REQUEST is not None:
            return self.manage_ip_lists()

    security.declareProtected(manage_users, 'getInfo')
    def getInfo(self):
        """ Returns info used in manage template """
        net_list = IP_ENABLED and [ip.strNormal() for ip in self._net_list] or [ip[0] for ip in self._net_list]
            
        return {'ip_list' : u"\n".join(self._ip_list),
                'net_list' : u"\n".join(net_list),
                'IP_ENABLED' : IP_ENABLED,
                'DISABLE' : DISABLE,
                'ALLOWED' : u"\n".join(ALLOWED) }

classImplements(CheckIPPlugin, IAuthenticationPlugin)
InitializeClass(CheckIPPlugin)
